#!/usr/bin/python

"""
	This file mainly exists for storing your public/private
	keypair in a location that's not tracked through Git. To
	run the myGengo test suite, fill in the values below and copy
	this file over to mygengo_test_keys.py.

	e.g: cp mygengo_test_keys_example.py mygengo_test_keys.py
"""

public_key = ''
private_key = ''
