#!/usr/bin/python

"""
	This file mainly exists for storing your public/private
	keypair in a location that's not tracked through Git. To
	run the pyGengo test suite, fill in the values below and copy
	this file over to pygengo_test_keys.py.

	e.g: cp pygengo_test_keys_example.py pygengo_test_keys.py
"""
#public_key='w5|ABkLj]DOko)3(B_Tw1ipILO_#l04d}zhHJaN3=jIPU^mlxvmpZN(@B_pIs~LB'
#private_key='gNtQ4k4JEu$zPY8qQ0zTvdV-RAf47lTGVblwo~wt@wiuDK[$qZ=DJ-4PZZ$nsEYs'
public_key='~8IuuuEQmqm3OM}vy5PSXzMXTUg=uezN^gT]}zoP_Fd7D#8KE#ceo#)PutT|9qZ1'
private_key='X[5pmR[I0pMZy(C]SrllsEaWm8Vd^kq-s1uvsm-|GPIJHhJ64cu#q~nk1XWT3F@2'
